/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Martin Borho <martin@borho.net>
                Raúl Yeguas <neokore@gmail.com>
                Riccardo Padovani <ricki.padovani@gmail.com>
 */
import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "CountryCodes.js" as Countries

Component {
    id: locationManagerComponent

    ComposerSheet {
        id: locationManagerSheet
        objectName: "LocationManagerSheet"
        title: i18n.tr("Edit locations")
        contentsHeight: parent.height

        property int initial_sum: 0

        Component.onCompleted: {
            locationsRemoved.clear();
            loadData();
            Theme.palette.selected.backgroundText = "#656565"
        }

        function loadData() {
            storage.getLocations(fillLocationsList);
        }

        function checkLocationExists(location) {
            for(var x=0;x<locationModel.count;x++) {
                var loc = locationModel.get(x);
                if(loc.service === location.service
                        && loc.service_id === location.service_id) {
                    existsNotification.show()
                    return true;
                }
            }
            return false;
        }

        function fillLocationsList(locations) {
            initial_sum = locations.length;
            locationModel.clear()
            for(var x=0;x<locations.length;x++) {
                var dbId = locations[x].db.id,
                    location = locations[x].location;
                // add db-id to the location object
                location.dbId = dbId;
                locationModel.append(locations[x].location);
            }
        }

        function addLocation(location) {
            if(!checkLocationExists(location)) {
                locationModel.append(location)
            }
        }

        container: Column {
            anchors.fill: parent
            Rectangle {
                width: parent.width
                height: units.gu(5)
                color: "transparent"
                Label {
                    text: " "+i18n.tr("Current location")
                    anchors {
                        verticalCenter: parent.verticalCenter
                        left: parent.left
                        leftMargin: units.gu(1)
                    }
                    fontSize: "medium"
                }
            }
            Rectangle {
                color: "#A9A9A9"
                width:parent.width
                height: units.gu(0.1)
            }
            ListItem.Standard {
                id: locationLookupItem
                progression: true
                Label {
                    text: i18n.tr("Lookup location")+" (Dummy)"
                    anchors {
                        verticalCenter: parent.verticalCenter
                        left: parent.left
                        leftMargin: units.gu(3)
                    }
                    fontSize: "large"
                }
                onClicked: {
                    print("TODO: lookup current location")
                    visible = false
                    currentLocationItem.visible = true;
                }
            }
            ListItem.Standard {
                id: currentLocationItem
                visible: false
                Label {
                    text: "London"
                    anchors {
                        verticalCenter: parent.verticalCenter
                        left: parent.left
                        leftMargin: units.gu(3)
                    }
                    fontSize: "large"
                }
                control: Button {
                    objectName: "currentLocationItem"
                    id: lookupItemAddButton
                    anchors.verticalCenter: parent.verticalCenter
                    text: i18n.tr("add")
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            // add dummy location
                            addLocation({"service_id":2643743,"name":"London","country":"GB","service":"openweathermap"})
                            lookupItemAddButton.visible = false;
                        }
                    }
                }
            }
            Rectangle {
                width: parent.width
                height: units.gu(5)
                color: "transparent"
                Label {
                    text: " "+i18n.tr("World")
                    anchors {
                        verticalCenter: parent.verticalCenter
                        left: parent.left
                        leftMargin: units.gu(1)
                    }
                    fontSize: "medium"
                }
            }
            Rectangle {
                color: "#A9A9A9"
                width:parent.width
                height: units.gu(0.1)
            }
            Rectangle {
                width: parent.width
                clip:true
                color:"transparent"
                height: locationManagerSheet.height-locationLookupItem.height-units.gu(19.2)
                ListView {
                    objectName: "LocationList"
                    anchors.fill: parent
                    model: locationModel
                    delegate: ListItem.Standard {
                        objectName: "existingLocationItem" + index
                        Label {
                            id: searchResultName
                            objectName: "existingLocation" + index
                            text: name
                            anchors {
                                top: parent.top
                                topMargin: units.gu(0.5)
                                left: parent.left
                                leftMargin: units.gu(3)
                            }
                            fontSize: "large"
                        }
                        Label {
                            text: (Countries.codes[country]) ? i18n.tr(Countries.codes[country]): country
                            anchors {
                                top: searchResultName.bottom
                                bottom: parent.bottom
                                left: parent.left
                                leftMargin: units.gu(3)
                            }
                            fontSize: "small"
                        }
                        removable: true
                        onItemRemoved: {
                            var location = locationModel.get(index)
                            // clone location obj
                            location = JSON.parse(JSON.stringify(location));
                            locationsRemoved.append(location)
                            locationModel.remove(index)

                        }
                        backgroundIndicator: SwipeDelete {
                            id: swipeDelete
                            state: swipingState
                        }
                    }
                    footer: ListItem.Standard {
                        objectName: "AddCityListItem"
                        progression: true
                        Label {
                            text: i18n.tr("Add city")
                            anchors {
                                verticalCenter: parent.verticalCenter
                                left: parent.left
                                leftMargin: units.gu(3)
                            }
                            fontSize: "large"
                        }
                        onClicked: {
                            PopupUtils.open(addLocationSheet)
                        }
                    }
                }
            }
        }

        Dialog {
            id:existsNotification
            title: i18n.tr("Location already added.")
            Button {
                text: i18n.tr("OK")
                onClicked: PopupUtils.close(existsNotification)
            }
        }

        ListModel {
            id: locationModel
        }

        ListModel {
            id: locationsRemoved
        }

        onConfirmClicked: {
            var locationsChanged = false;
            // remove
            for(var x=0;x<locationsRemoved.count;x++) {
                var delLoc = locationsRemoved.get(x);
                storage.clearLocation(delLoc.dbId);
                locationsChanged = true;
            }
            // add
            for(var y=0;y<locationModel.count;y++) {
                var loc = locationModel.get(y);
                if(loc.dbId === undefined || loc.dbId=== 0) {
                    storage.insertLocation({location:loc});
                    locationsChanged = true;
                }
            }
            // refresh tabs if neccessary
            if(locationsChanged || locationModel.count === 0) {
                mainView.refreshData()
            }
            // handle dummy "current location" item
            currentLocationItem.visible = false
            locationLookupItem.visible = true
            lookupItemAddButton.visible = true
            Theme.palette.selected.backgroundText = "#f4f4e8"
        }

        onCancelClicked: {
            if(initial_sum === 0) {
                // refresh mainview to land again here,
                // when no locations were defined when
                // opening the sheet
                mainView.refreshData()
            }
            Theme.palette.selected.backgroundText = "#f4f4e8"
        }

        AddLocationSheet {
            id:addLocationSheet
        }
    }
}
