# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for managing locations in Weather app"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Is, Not
from autopilot.matchers import Eventually

from ubuntu_weather_app.tests import WeatherTestCase, DatabaseMixin, SheetMixin
from ubuntu_weather_app.tests.weatherdata import locations_data

from time import sleep


class TestLocationManager(WeatherTestCase, DatabaseMixin, SheetMixin):
    def setUp(self):
        self.clean_db()
        super(TestLocationManager, self).setUp()
        self.assertThat(
            self.get_qml_view().visible, Eventually(Equals(True)))

    def search_for_city(self, city):
        self.assertThat(lambda: self.main_window.get_object('TextField', 'SearchField'), Eventually(Not(Is(None))))
        searchField = self.main_window.get_object("TextField", "SearchField")
        #poll clicking for focus for lack of better method, due to screen switching animation
        #is there a property we can use instead?
        timeout = 0
        while searchField.focus != True and timeout < 10:
            self.pointing_device.click_object(searchField)
            sleep(1)
            timeout += 1
        self.keyboard.type(city)
        self.assertThat(searchField.text, Eventually(Equals(city)))
        self.keyboard.press_and_release('Enter')
        return searchField

    def _open_add_location_page(self):
        """Opens the AddLocationPage"""
        # click on listitem to open AddLocationPage
        self.assertThat(lambda: self.main_window.get_object('QQuickListView', 'LocationList'), Eventually(Not(Is(None))))
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        addCityItem = locationList.get_children()[0]
        self.assertThat(lambda: addCityItem.visible, Eventually(Equals(True)))
        self.pointing_device.move_to_object(addCityItem)
        self.pointing_device.click()

        addLocPage = self.main_window.get_object("DefaultSheet", "AddLocationSheet")
        self.assertThat(addLocPage.visible, Eventually(Equals(True)))

    def select_single_retry(self, object_type, **kwargs):
        """Returns the item that is searched for with app.select_single
        In case of the item was not found (not created yet) a second attempt is
        taken 1 second later."""
        item = self.app.select_single(object_type, **kwargs)
        tries = 10
        while item is None and tries > 0:
            sleep(0.2)
            item = self.app.select_single(object_type, **kwargs)
            tries = tries - 1
        return item

    def test_add_location(self):
        """Adds a location"""
        self._open_add_location_page()

        # insert city name to search for
        self.search_for_city("London")

        # wait for results and click on the first item
        http_failed = lambda: self.main_window.get_object('Dialog', 'HTTPFailedDialog').visible
        self.assertThat(http_failed, Eventually(NotEquals(True)))
        load_indicator = self.main_window.get_object('ActivityIndicator', 'SearchingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False), timeout=120))
        resultsList = self.main_window.get_object('QQuickListView', 'SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        self.assertThat(lambda: self.main_window.get_object('Label', 'searchResult0'), Eventually(Not(Is(None))))
        firstResult = self.main_window.get_object('Label', 'searchResult0')
        self.pointing_device.move_to_object(firstResult)
        self.pointing_device.click()

        # LocationManagerPage should be visible and "London" added
        self.assertThat(lambda: self.main_window.get_object('QQuickListView', 'LocationList'), Eventually(Not(Is(None))))
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        self.assertThat(lambda: self.main_window.get_object('Label', 'existingLocation0'), Eventually(Not(Is(None))))
        addedItem = self.main_window.get_object('Label', "existingLocation0")
        self.assertThat(addedItem.text, Eventually(Equals("London")))
        self._click_sheet_confirm()

        # back to locations, wait till data is loaded
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False), timeout=120))

        # only location is there
        loadingPage = self.main_window.get_object("Tabs", "rootTabs")
        self.assertThat(loadingPage.visible, Eventually(Equals(True)))
        tabObjects = self.main_window.get_objects('LocationTab','LocationTab')
        self.assertEqual(1, len(tabObjects))

    def test_add_predefined_location(self):
        """Adds a location"""
        self._open_add_location_page()

        resultsList = self.main_window.get_object('QQuickListView', 'SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        self.assertThat(lambda: self.main_window.get_object('Label', 'searchResult4'), Eventually(Not(Is(None))))
        firstResult = self.main_window.get_object('Label', 'searchResult4')
        self.pointing_device.move_to_object(firstResult)
        self.pointing_device.click()

        # LocationManagerPage should be visible and "London" added
        self.assertThat(lambda: self.main_window.get_object('QQuickListView', 'LocationList'), Eventually(Not(Is(None))))
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        self.assertThat(lambda: self.main_window.get_object('Label', 'existingLocation0'), Eventually(Not(Is(None))))
        addedItem = self.main_window.get_object('Label', "existingLocation0")
        self.assertThat(addedItem.text, Eventually(Equals("Hamburg")))
        self._click_sheet_confirm()

        # back to locations, wait till data is loaded
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False), timeout=120))

        # only location is there
        loadingPage = self.main_window.get_object("Tabs", "rootTabs")
        self.assertThat(loadingPage.visible, Eventually(Equals(True)))
        tabObjects = self.main_window.get_objects('LocationTab','LocationTab')
        self.assertEqual(1, len(tabObjects))

    def test_search_city(self):
        """Search for cities"""
        self._open_add_location_page()

        # insert city name to search for
        searchField = self.search_for_city("London")

        # wait for results and click on the first item
        http_failed = lambda: self.main_window.get_object('Dialog', 'HTTPFailedDialog').visible
        self.assertThat(http_failed, Eventually(NotEquals(True)))
        load_indicator = self.main_window.get_object('ActivityIndicator', 'SearchingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False), timeout=120))
        resultsList = self.main_window.get_object('QQuickListView', 'SearchResultList')
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        self.assertThat(lambda: self.main_window.get_object('Label', 'searchResult0'), Eventually(Not(Is(None))))
        firstResult = self.select_single_retry("Label", objectName="searchResult0")
        self.assertThat(firstResult.text, Eventually(Equals("London")))

        # regain focus, clear search field and do another search
        self.assertThat(lambda: self.main_window.get_object('TextField', 'SearchField'), Eventually(Not(Is(None))))
        searchField = self.main_window.get_object("TextField", "SearchField")
        self.pointing_device.click_object(searchField)
        self.assertThat(lambda: self.main_window.select_single('AbstractButton'), Eventually(Not(Is(None))))
        clear_button = searchField.select_single("AbstractButton")
        self.pointing_device.click_object(clear_button)
        self.assertThat(searchField.text, Eventually(Equals("")))

        self.pointing_device.click_object(searchField)
        self.keyboard.type("Hamburg")
        self.keyboard.press_and_release('Enter')
        load_indicator = self.main_window.get_object('ActivityIndicator', 'SearchingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False), timeout=120))
        self.assertThat(resultsList.visible, Eventually(Equals(True)))
        self.assertThat(lambda: self.main_window.get_object('Label', 'searchResult0'), Eventually(Not(Is(None))))
        firstResult = self.select_single_retry("Label", objectName="searchResult0")
        self.assertThat(firstResult.text, Eventually(Equals("Hamburg")))

    def test_cancel_adding_location(self):
        """Cancel the cities search"""
        self._open_add_location_page()
        self._click_sheet_cancel()
        self.assertThat(lambda: self.main_window.get_object('QQuickListView', 'LocationList'), Eventually(Not(Is(None))))
        locationList = self.main_window.get_object('QQuickListView', 'LocationList')
        self.assertThat(locationList.visible, Eventually(Equals(True)))

    def test_no_location_found(self):
        """Tests if label with 'No city found' is shown """
        self._open_add_location_page()

        # insert city name to search for
        self.search_for_city("UbuntuCity")

        # wait for result and look if label is shown
        http_failed = lambda: self.main_window.get_object('Dialog', 'HTTPFailedDialog').visible
        self.assertThat(http_failed, Eventually(NotEquals(True)))
        load_indicator = self.main_window.get_object('ActivityIndicator', 'SearchingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False), timeout=120))
        errorLabel = self.main_window.get_object('Label', 'noCityError')
        self.assertThat(errorLabel.visible, Eventually(Equals(True)))


class TestLocationManagerWithLocation(WeatherTestCase, DatabaseMixin, SheetMixin):
    def setUp(self):
        self.clean_db()
        self.launch_and_quit_app()
        # add one location to storage
        self.save_locations_to_storage(locations_data)
        super(TestLocationManagerWithLocation, self).setUp()
        self.assertThat(
            self.get_qml_view().visible, Eventually(Equals(True)))

    def _open_location_manager(self):
        """Opens the location manager"""
        self.main_window.open_toolbar().click_button("EditButton")

    def _swipe_location_to_remove(self):
        """Swipe right to delete the first location"""
        number_of_locations = self._get_number_of_locations()
        self.assertThat(lambda: self.app.select_single('Standard', objectName='existingLocationItem0'), Eventually(Not(Is(None))))
        firstLocation = self.app.select_single("Standard", objectName="existingLocationItem0")

        x, y, w, h = firstLocation.globalRect
        tx = x + (w / 8)
        ty = y + (h / 2)
        self.pointing_device.drag(tx, ty, tx + (w - w / 8), ty)
        self.assertThat(
            self._get_number_of_locations(),
            Eventually(Equals(number_of_locations-1)))

    def _get_number_of_locations(self):
        qqlw = self.app.select_single("ComposerSheet").select_single("QQuickListView")
        return qqlw.count

    def test_remove_location(self):
        """Removes location"""
        # wait till data is loaded
        loadingPage = self.main_window.get_object("Tabs", "rootTabs")
        self.assertThat(loadingPage.visible, Eventually(Equals(True)))

        # count locations at start
        tabObjects = self.main_window.get_objects('LocationTab','LocationTab')
        tabsSumStart = len(tabObjects)

        # go to the location manager and remove location
        self._open_location_manager()
        self._swipe_location_to_remove()
        self._click_sheet_confirm()

        # back to locations, only one is left
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False)))
        tabObjects = self.main_window.get_objects('LocationTab','LocationTab')
        self.assertThat(lambda: tabsSumStart-1, Eventually(Equals(len(tabObjects))))

    def test_cancel_remove_location(self):
        """Cancels removing of location"""
        # wait data is loaded
        loadingPage = self.main_window.get_object("Tabs", "rootTabs")
        self.assertThat(loadingPage.visible, Eventually(Equals(True)))

        # count locations at start
        tabObjects = self.main_window.get_objects('LocationTab','LocationTab')
        tabsSumStart = len(tabObjects)

        # go to the location manager, remove location, but click cancel
        self._open_location_manager()
        self._swipe_location_to_remove()
        self._click_sheet_cancel()

        # back to locations, no loction is removed
        loadingPage = self.main_window.get_object("Tabs", "rootTabs")
        self.assertThat(loadingPage.visible, Eventually(Equals(True)))
        tabObjects = self.main_window.get_objects('LocationTab','LocationTab')
        self.assertEqual(tabsSumStart, len(tabObjects))

