# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests for managing locations in Weather app"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals, Is, Not
from autopilot.matchers import Eventually

from ubuntu_weather_app.tests import WeatherTestCase, DatabaseMixin, SheetMixin
from ubuntu_weather_app.tests.weatherdata import locations_data

class TestSettings(WeatherTestCase, DatabaseMixin, SheetMixin):
    def setUp(self):
        self.clean_db()
        self.launch_and_quit_app()
        self.save_locations_to_storage(locations_data)
        super(TestSettings, self).setUp()
        self.assertThat(
            self.get_qml_view().visible, Eventually(Equals(True)))

    def _open_settings_sheet(self):
        """Opens the settings sheet"""
        self.main_window.open_toolbar().click_button("SettingsButton")

    def _move_pointer_around(self):
        """Helper method to move the pointer around, to assure selector is opened"""
        self.assertThat(lambda: self.main_window.get_object('ComposerSheet', 'SettingsSheet'), Eventually(Not(Is(None))))
        sheet = self.main_window.get_object('ComposerSheet', 'SettingsSheet')
        self.pointing_device.move_to_object(sheet)

    def _check_units(self, units):
        """Checks selected units by values from the first location tab"""
        self.assertThat(lambda: self.main_window.get_objects('QQuickText', 'CurrentTempText'), Eventually(Not(Is(None))))
        current_temps = self.main_window.get_objects('QQuickText', 'CurrentTempText')
        if units == "imperial":
            self.assertThat(current_temps[0].text, Eventually(Equals(u'72')))
            self.assertThat(current_temps[1].text, Eventually(Equals(u'74')))
        else:
            self.assertThat(current_temps[0].text, Eventually(Equals(u'22')))
            self.assertThat(current_temps[1].text, Eventually(Equals(u'24')))

    def _check_wind_units(self, wind_units):
        """Checks selected units by values from the first location tab"""
        self.assertThat(lambda: self.main_window.get_objects('WeatherDetailComponent', 'WindSpeedValue')[0], Eventually(Not(Is(None))))
        wind_component = self.main_window.get_objects('WeatherDetailComponent', 'WindSpeedValue')[0]
        self.assertThat(lambda: wind_component.get_object('QQuickText', 'WeatherDetailUnit'), Eventually(Not(Is(None))))
        wind_unit = wind_component.get_object('QQuickText', 'WeatherDetailUnit')
        self.assertThat(lambda: wind_component.get_object('QQuickText', 'WeatherDetailValue'), Eventually(Not(Is(None))))
        wind_value = wind_component.get_object('QQuickText', 'WeatherDetailValue')
        if wind_units == "mph":
            self.assertThat(wind_unit.text, Eventually(Equals(u'mph NW')))
            self.assertThat(wind_value.text, Eventually(Equals(u'3')))
        else:
            self.assertThat(wind_unit.text, Eventually(Equals(u'km/h NW')))
            self.assertThat(wind_value.text, Eventually(Equals(u'4')))


    def test_switch_scale(self):
        """Tests switching the scale in the settings"""
        # first check metric values and open the settings sheet
        self._check_units('metric')
        self._open_settings_sheet()

        # open the value selector
        self.assertThat(lambda: self.main_window.get_object('OptionSelector', 'UnitsSelector'), Eventually(Not(Is(None))))
        units_selector = self.main_window.get_object('OptionSelector', "UnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many('ShapeItem')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, fahrenheit
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[2], Eventually(Not(Is(None))))
        imperial_option = units_selector.select_many('ShapeItem')[2]
        self.pointing_device.click_object(imperial_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self._click_sheet_confirm()

        # wait for reload and check the imperial values
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False)))
        self._check_units('imperial')

        # switch back to metric values again
        self._open_settings_sheet()
        self.assertThat(lambda: self.main_window.get_object('OptionSelector', 'UnitsSelector'), Eventually(Not(Is(None))))
        units_selector = self.main_window.get_object('OptionSelector', "UnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[2], Eventually(Not(Is(None))))
        current_option = units_selector.select_many('ShapeItem')[2]
        self.pointing_device.click_object(current_option)
        # while it's opened, units_selector returns false
        # self.assertThat(units_selector.expanded, Eventually(Equals(True)))

        # click celsius option
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[1], Eventually(Not(Is(None))))
        metric_option = units_selector.select_many('ShapeItem')[1]
        self.pointing_device.click_object(metric_option)

        # confirm
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self._click_sheet_confirm()

        # wait for reload and check the metric values again
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False)))
        self._check_units('metric')

    def test_switch_scale_cancel(self):
        """Test canceling switching scale"""
        # first check metric values and open the settings sheet
        self._check_units('metric')
        self._check_wind_units('kmh')
        self._open_settings_sheet()

        # open the temp value selector
        self.assertThat(lambda: self.main_window.get_object('OptionSelector', 'UnitsSelector'), Eventually(Not(Is(None))))
        units_selector = self.main_window.get_object('OptionSelector', "UnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many('ShapeItem')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, fahrenheit
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[2], Eventually(Not(Is(None))))
        imperial_option = units_selector.select_many('ShapeItem')[2]
        self.pointing_device.click_object(imperial_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))

        # open the wind value selector
        self.assertThat(lambda: self.main_window.get_object('OptionSelector', 'WindUnitsSelector'), Eventually(Not(Is(None))))
        units_selector = self.main_window.get_object('OptionSelector', "WindUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.pointing_device.click_object(units_selector)

        # choose second option, mph
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[2], Eventually(Not(Is(None))))
        mph_option = units_selector.select_many('ShapeItem')[2]
        self.pointing_device.click_object(mph_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))

        # cancel and check
        self._click_sheet_cancel()
        self._check_units('metric')
        self._check_wind_units('kmh')

    def test_switch_wind_scale(self):
        """Tests switching the wind scale in the settings"""
        # first check metric values and open the settings sheet
        self._check_wind_units('kmh')
        self._open_settings_sheet()

        # open the wind value selector
        self.assertThat(lambda: self.main_window.get_object('OptionSelector', 'WindUnitsSelector'), Eventually(Not(Is(None))))
        units_selector = self.main_window.get_object('OptionSelector', "WindUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[1], Eventually(Not(Is(None))))
        current_option = units_selector.select_many('ShapeItem')[1]
        self.pointing_device.click_object(current_option)

        # choose second option, mph
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[2], Eventually(Not(Is(None))))
        mph_option = units_selector.select_many('ShapeItem')[2]
        self.pointing_device.click_object(mph_option)
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self._click_sheet_confirm()

        # wait for reload and check the mph values
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False)))
        self._check_wind_units('mph')

        # switch back to kmh values again
        self._open_settings_sheet()
        self.assertThat(lambda: self.main_window.get_object('OptionSelector', 'WindUnitsSelector'), Eventually(Not(Is(None))))
        units_selector = self.main_window.get_object('OptionSelector', "WindUnitsSelector")
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(1)))
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[2], Eventually(Not(Is(None))))
        current_option = units_selector.select_many('ShapeItem')[2]
        self.pointing_device.click_object(current_option)
        # while it's opened, units_selector returns false
        # self.assertThat(units_selector.expanded, Eventually(Equals(True)))

        # click celsius option
        self._move_pointer_around()
        self.assertThat(lambda: units_selector.select_many('ShapeItem')[1], Eventually(Not(Is(None))))
        metric_option = units_selector.select_many('ShapeItem')[1]
        self.pointing_device.click_object(metric_option)

        # confirm
        self.assertThat(units_selector.selectedIndex, Eventually(Equals(0)))
        self._click_sheet_confirm()

        # wait for reload and check the metric values again
        load_indicator = self.main_window.get_object('ActivityIndicator', 'LoadingSpinner')
        self.assertThat(load_indicator.running, Eventually(Equals(False)))
        self._check_units('metric')

