/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Raúl Yeguas <neokore@gmail.com>
 *              Martin Borho <martin@borho.net>
 */
import QtQuick 2.0
import Ubuntu.Components 0.1
import "components" as Components
import "components/GradientsMap.js" as Gradients
import Ubuntu.Components.Popups 0.1

MainView {
    // objectName for functional testing purposes (autopilot-qt5)
    objectName: "weather"
    applicationName: "ubuntu-weather-app"

    id: mainView
    
    width: units.gu(50)
    height: units.gu(75)

    headerColor: "#57365E"
    backgroundColor: "#A55263"
    footerColor: "#D75669"

    property string newHeaderColor: "#57365E"
    property string newBackgroundColor: "#A55263"
    property string newFooterColor: "#D75669"

    property var locationsList: []
    property var tabsObject: null
    // set default values for settings here
    property var settings: {
        "units": Locale.MetricSystem === 1 ? "metric" : "imperial",
        "wind_units": Locale.MetricSystem === 1 ? "kmh" : "mph"
    }

    WorkerScript {
        id: locationDataWorker
        source: "components/WeatherApi.js"
        onMessage: {
            if(!messageObject.error) {
                if(messageObject.action === "updateData") {
                    messageObject.result.forEach(function(loc) {
                        if(loc["save"] === true) {
                            storage.updateLocation(loc.db.id, loc);
                        }
                    });
                    buildTabs(messageObject.result);
                }
            } else {
                console.log(messageObject.error.msg+" / "+messageObject.error.request.url)
                httpFailedDialog.show();
                refreshData(true, null);
            }
        }
    }

    ParallelAnimation {
        id: gradientAnimation
        ColorAnimation {
            target: mainView
            property: "headerColor"
            from: headerColor
            to: newHeaderColor
            duration:UbuntuAnimation.BriskDuration
            easing: UbuntuAnimation.StandardEasing
        }
        ColorAnimation {
            target: mainView
            property: "backgroundColor"
            from: backgroundColor
            to: newBackgroundColor
            duration:UbuntuAnimation.BriskDuration
            easing: UbuntuAnimation.StandardEasing
        }
        ColorAnimation {
            target: mainView
            property: "footerColor"
            from: footerColor
            to: newFooterColor
            duration:UbuntuAnimation.BriskDuration
            easing: UbuntuAnimation.StandardEasing
        }
    }

    function adjustBackground(degrees) {
        gradientAnimation.stop()
        var celsius = (settings.units === "imperial") ? Math.round((degrees-32)/1.8) : degrees,
            colors = Gradients.getGradients(celsius);
        newHeaderColor = colors.header;
        newBackgroundColor = colors.body;
        newFooterColor = colors.footer;
        gradientAnimation.start()
    }

    // see https://bugs.launchpad.net/ubuntu-ui-toolkit/+bug/1124071, temporary workaround
    function buildTabs(locations) {
        // check if there was a location added
        var locLength = locations.length,
            locBeforeLen = locationsList.length,
            focusToLast = (locBeforeLen > 0 && locLength > locBeforeLen) ? true : false;
        refresh.visible = false;
        // show locationmanager when no location is added
        if(locLength === 0) {
            showLocationManager()
            return;
        }

        locationsList = locations;        
        if(tabsObject !== null) {
            tabsObject.destroy()
        }
        var tabsString = "import QtQuick 2.0; import Ubuntu.Components 0.1; import Ubuntu.Components.Popups 0.1; import \"components\" as Components; "
                + "Tabs {id: tabs; objectName: 'rootTabs'; anchors.fill: parent;"
        if(locLength > 0) {
            for(var x=0;x<locLength;x++) {
                tabsString += "Components.LocationTab {title: '"+locations[x].location.name+"'; locationIndex: "+x+"} "
            };
        }
        tabsString += "}"; // END Tabs componen
        tabsObject = Qt.createQmlObject(tabsString, tabPage, "tabs")
        if(focusToLast)
            tabsObject.selectedTabIndex = locLength -1;
    }

    function refreshData(from_storage, force_refresh) {
        refresh.visible = true;
        if(from_storage === true && force_refresh !== true) {
            storage.getLocations(buildTabs);
        } else {
            storage.getLocations(function(locations) {
                locationDataWorker.sendMessage({
                    action: "updateData",
                    params: {locations:locations, force: force_refresh}
                })
            });
        }
    }

    function showLocationManager() {
        PopupUtils.open(locationManager)
    }

    function showSettings() {
        PopupUtils.open(settingsSheet)
    }

    Component.onCompleted: {
        //storage.clearDB();
        //storage.clearSetting('units');
        storage.getSettings(function(storedSettings) {
            for(var settingName in storedSettings) {
                settings[settingName] = storedSettings[settingName];
            }
            refreshData();
        })
    }

    Components.SettingsSheet {
        id: settingsSheet
    }
    
    Components.Storage{
        id: storage
    }    

    Components.LocationManagerSheet {
        id:locationManager        
    }

    UbuntuShape {
        id: refresh
        height: units.gu(5)
        width: units.gu(5)
        color: "#90FFFFFF"
        visible: false
        anchors {
            horizontalCenter: parent.horizontalCenter
            top: parent.top
            margins: units.gu(15)
        }

        ActivityIndicator {
            id: loading
            objectName: "LoadingSpinner"
            anchors.centerIn: parent
            running: refresh.visible
            z: 1
        }
    }

    Item {
        id:tabPage
        width: parent.width
        height:parent.height
    }

    Dialog {
        id:httpFailedDialog
        objectName: "HTTPFailedDialog"
        title: i18n.tr("Something went wrong!")
        text: i18n.tr("Please make sure, you are connected to the internet.")
        z:10
        Button {
            text: i18n.tr("OK")
            onClicked: PopupUtils.close(httpFailedDialog)
        }
    }


}
